import java.util.*;
import java.io.*;

class Exec {

    public final static String PERL = "#!/s/std/bin/perl";
    public final static String CSH = "#! /bin/csh -f";
    public final static String SQLENGINE = "SQL";
    public final static String PGENGINE = "PG";
    public final static String MRENGINE = "MR";
    
    public final static String INSERTS_SQL = "inserts.sql";
    public final static String INSERTS_ERR = "inserts.err";
    public final static String CREATE_SQL  = "create.sql";

    //    private String dbname;
    
    public static String dbtitle;

    /*
    public final static String MINIREL_DIR =
	"/u/m/d/md/Coursework/736/P/minirel/";
    */

    static BufferedReader in = null;

    Exec() {}
    
    public void execDB(String dbname, String fileName, String tableName,
		       BufferedReader inputs) throws IOException {
	if (dbname.equals("PG")) {
	    execPG(fileName,tableName,inputs);
	}
	else if (dbname.equals("MR")) {
	    execMR(fileName,tableName,inputs);
	}
	else if (dbname.equals("SQL")) {
	    execSQL(fileName,tableName,inputs);
	}
	else {
	    U.println("Not connected");
	    return;
	}
    }

    public void execSQL(String fileName, String tableName,
			BufferedReader inputBuffer) 
	throws IOException {

	dbtitle = "bduphoff";
	SQLEngine sql = new SQLEngine();
	generateInputs(sql, fileName, tableName, inputBuffer);

	Command cmd = new Command("isql -Ubduphoff -P -i " +
				  CREATE_SQL);
	cmd.run();
	Command cmd2 = new Command("isql -Ubduphoff -P -i " +
				  INSERTS_SQL);
	cmd2.run();
	dbtitle = "md";
    }

    public void generateInputs(Engine eng, 
			       String fileName, String tableName,
			       BufferedReader bfr) 
	throws IOException { 
	
	PrintWriter out = new PrintWriter(new FileWriter(INSERTS_SQL));
	PrintWriter err = new PrintWriter(new FileWriter(INSERTS_ERR));
	PrintWriter createOut = new PrintWriter(new FileWriter(CREATE_SQL));
	SchemaDefinition sd = new SchemaDefinition(fileName);
	sd.setTableName(tableName);
	String createTable = eng.getCreateTable(sd);
	createOut.println(createTable);
	createOut.println(eng.quit());
	createOut.close();
	String str = bfr.readLine();
	int count = 1;
	while (str != null) {
	    String inserts = eng.getInsert(sd, str);

	    if (inserts == null)
		err.println(str);
	    else 
		out.println(inserts);
	    str = bfr.readLine();
	    count++;
	}
	out.println(eng.quit());
	out.close();
	err.close();
	bfr.close();
    }
    
    public void execPG(String fileName, String tableName, 
		       BufferedReader inputs) 
	throws IOException {

	PostgresEngine pg = new PostgresEngine();
	generateInputs(pg, fileName, tableName, inputs);
	
	Command cmd =
	    new Command("psql -q -f " + CREATE_SQL + U.SPACE + 
			dbtitle);
	cmd.run();
	// delay a bit until database table is ready
	
	
	try {
	    Thread.sleep(5000);
	} catch (InterruptedException e) {
	    U.println("ExecPG InterruptedEXCEPTION " + e.getMessage());
	}
	
	//proceed with the insertions
	Command cmd2 =
	    new Command("psql -q -f " + INSERTS_SQL + U.SPACE +
			dbtitle);
	cmd2.run();

    } //  Postgres
    
    public void execMR(String fileName, String tableName,
		       BufferedReader inputs)
	throws IOException {
	
	MinirelEngine mr = new MinirelEngine();
	generateInputs(mr,fileName, tableName, inputs);

	try {
	    Thread.sleep(5000);
	} catch (InterruptedException e) {
	    U.println("ExecPG InterruptedEXCEPTION " + e.getMessage());
	}
	/*
	Command cmd =
	    new Command("minirel " + dbtitle + U.SPACE +
			U.REDIRECT + U.SPACE + CREATE_SQL);
	*/
	PrintWriter pw = new PrintWriter(new FileWriter("execMR.csh"));
	Command cmd0 = new Command("chmod u+x execMR.csh");
	cmd0.run();

	pw.println(CSH);
	pw.println();
	pw.println("minirel " + dbtitle + U.REDIRECT + CREATE_SQL);
	pw.println("sleep 1");
	pw.println("minirel " + dbtitle + U.REDIRECT + INSERTS_SQL);

	pw.close();
	Command cmd = new Command("execMR.csh");
	cmd.run();
	    // delay a bit until database table is ready
	    /*
	    cmd.printResults();
	    try {
		Thread.sleep(500);
	    } catch (InterruptedException e) {
		U.println("ExecPG InterruptedEXCEPTION " + e.getMessage());
	    }
	    */
	    //proceed with the insertions
	    /*    
	    Command cmd2 =
		new Command("minirel" + U.SPACE +
			    dbtitle + U.SPACE +
			    U.REDIRECT + U.SPACE + INSERTS_SQL);
	    cmd2.run();
	    cmd2.printResults();
	    */
    } //  Minirel

    public static void populateDB(String dbname, String fileName,
				  String tableName, 
				  BufferedReader inputs) {
	try {
	    Exec ex = new Exec();
	    ex.execDB(dbname, fileName, tableName, inputs);
	} catch (IOException e) {
	    e.printStackTrace();
	}
    }
}





