import java.util.*;

public class Field {
    private int minInLen = 1;
    private int maxInLen;
    private boolean isVariable;
    private String delim = "";
    private String name = "";
    private String inType = "";
    private boolean isIncluded = true;
    
    // The following variables are for value mapping only
    private int maxOutLen;
    private String defaultValue = "";
    private String outType = "";
    private Vector inVals = new Vector();
    private Vector outVals = new Vector();
        
    public Field() {
    }


    public void setMinInLength(int min) {
	minInLen = min;
	if (minInLen <= 0)
	    minInLen = 1;
	else if (minInLen > 255)
	    minInLen = 255;
	
	if (minInLen > maxInLen)
	    maxInLen = minInLen;
    }


    public int getMinInLength() {
	return minInLen;
    }


    public void setMaxInLength(int max) {
	maxInLen = max;
	if (maxInLen > 255)
	    maxInLen = 255;
	else if (maxInLen <= 0)
	    maxInLen = 1;
	
	if (maxInLen < minInLen)
	    maxInLen = minInLen;
    }
    

    public int getMaxInLength() {
	return maxInLen;
    }


    public void setVariable(boolean isVariable) {
	this.isVariable = isVariable;
    }


    public boolean isVariable() {
	return isVariable;  
    }


    public void setDelimiter(String del) {
	delim = del;
	if (delim == null)
	    delim = "";
    }


    public String getDelimiter() {
	return delim;
    }


    public void setName(String fieldName) {
	name = fieldName;
    }


    public String getName() {
	return name;
    }

    public String getInType() {
	return inType;
    }

    public void setInType(String type) {
	inType = type;
    }

    public String getOutType() {
	if (outType.length() == 0)
	    return inType;
	return outType;
    }

    public void setOutType(String type) {
	outType = type;
    }

    public int getMaxOutLength() {
	if (maxOutLen == 0)
	    return maxInLen;
	return maxOutLen;
    }
    
    public void setMaxOutLength(int len) {
	maxOutLen = len;
    }

    public String getDefaultValue() {
	return defaultValue;
    }

    public void setDefaultValue(String value) {
	defaultValue = value;
    }

    public void setIncluded(boolean isIncluded) {
	this.isIncluded = isIncluded;
    }

    public boolean isIncluded() {
	return isIncluded;
    }

    public int addValueMapping(String inVal, String outVal) {
	// Return Value Reason
	// ============ ======
	//    -1        General error
	//    -2
	//    -3
	//    -4
	//    -5
	//    -6

	if (inType.compareTo("CHAR") == 0) {
	    if (inVal.length() > maxInLen)
		return -2;
	    if (inVal.length() < minInLen)
		return -3;
	}
	else if (inType.compareTo("INTEGER") == 0) {
	    int inValInt = 0;
	    try {
		inValInt = Integer.parseInt(inVal);
	    }
	    catch (NumberFormatException e) {
		return -5;
	    }
	}
	else if (inType.compareTo("REAL") == 0) {
	    
	}
	
	if (outType.compareTo("CHAR") == 0) {
	    if (outVal.length() > maxOutLen)
		return -4;
	}
	else if (outType.compareTo("INTEGER") == 0) {
	    int outValInt = 0;
	    try {
		outValInt = Integer.parseInt(outVal);
	    }
	    catch (NumberFormatException e2) {
		return -6;
	    }
	}
	else if (outType.compareTo("REAL") == 0) {
	    
	}
	inVals.addElement(inVal);
	outVals.addElement(outVal);
	return 0;
    }

    public int validate() {
	// Return Value  Reason
	// ============  ======
	//    -1         minInLen = 0
	//    -2         maxInLen = 0
	//    -3         minLen greater than maxLen
	//    -4         Variable len fields cannot have a null delimiter
	//    -5         Variable len fields cannot have a space in the delimiter
	//    -6         Input Type is not valid 
	//    -7         Output type is not valid

	if (minInLen == 0)
	    return -1;
	
	if (maxInLen == 0)
	    return -2;
	
	if (minInLen > maxInLen)
	    return -3;         
	
	if ((isVariable) && (delim == null))
	    return -4;
	
	if ((isVariable) && (delim.indexOf(" ") != -1))
	    return -5;
	
	boolean validType = false;
	if (inType.compareTo("CHAR") == 0)
	    validType = true;
	else if (inType.compareTo("INTEGER") == 0)
	    validType = true;
	else if (inType.compareTo("REAL") == 0)
	    validType = true;
	
	if (!validType)
	    return -6;

	if (maxOutLen > 0) {
	    validType = false;
	    if (outType.compareTo("CHAR") == 0)
		validType = true;
	    else if (outType.compareTo("INTEGER") == 0)
		validType = true;
	    else if (outType.compareTo("REAL") == 0)
		validType = true;
	    
	    if (!validType)
		return -7;
	}

	return 0;
    }

    public String toString() {
    	String d = ", "; 
    	return name + d + isVariable + d + minInLen + d + maxInLen + d + delim;
    }

    public String toXML() {
	String temp = "";
	temp += "<Field";
	temp += " name=\"" + name + "\" isVariable=\"" + isVariable + "\"";
	temp += " minInLength=\"" + minInLen + "\" maxInLength=\"" + maxInLen + "\"";
	temp += " maxOutLength=\"" + maxOutLen + "\"";
	temp += " delimiter=\"" + delim + "\" inType=\"" + inType + "\"";
	temp += " outType=\"" + outType + "\"";
	temp += " isIncluded=\"" + isIncluded + "\"";
	temp += " defaultValue=\"" + defaultValue + "\">";
	for (int i = 0; i < inVals.size(); i++) {
	    temp += "<Value inVal=\"" + (String)inVals.elementAt(i) + "\"";
	    temp += " outVal=\"" + (String)outVals.elementAt(i)+"\"";
            temp += "></Value>";
        }
        temp += "</Field>";
        return temp;
    }

    public boolean hasValueMapping() {
	if (outType.length() == 0)
	    return false;
	else
	    return true;
    }
    
    public String getMappedValue(String inVal) {
	String outVal = null;
	for (int i = 0; i < inVals.size(); i++) {
	    String val = (String)inVals.elementAt(i);
	    if (inVal.compareTo(val) == 0) {
		outVal = (String)outVals.elementAt(i);
		break;
	    }
	}
	if (outVal == null)
	    return null;
	
	// Check that values are correctly formatted
	if (outType.compareTo("INTEGER") == 0) {
	    try {
		int intVal = Integer.parseInt(outVal);
	    }
	    catch (NumberFormatException e) {
		return null;
	    }
	}
	else if (outType.compareTo("REAL") == 0) {
	    try {
		float floatVal = Float.parseFloat(outVal);
	    }
	    catch (NumberFormatException e) {
		return null;
	    }
	}
	else if (outType.compareTo("CHAR") == 0) {
	    if (outVal.length() > maxOutLen) 
		return null;
	}
	else
	    return null;
	
	return outVal;
    }
}
