import java.util.*;
import java.io.*;

class PostgresEngine extends Engine {
// Engine for creating Postgres compatable SQL

	public PostgresEngine() {

	}

    /* INSERT EXAMPLE */

    /*
      insert into employee
      (first, last, age, address, city,
      state)
      values ('Luke', 'Duke', 45, '2130
      Boars Nest', 'Hazard Co',
      'Georgia'); 
    */
  
    public String getInsert(SchemaDefinition sd, String data) {
	// Returns the SQL needed to insert the given data

		String temp = "INSERT INTO " + sd.getTableName() + U.SPACE +
		    U.LPAREN;
		for (int i = 0; i < sd.getNumFields(); i++) {
			temp += sd.getField(i).getName();
			if (i != sd.getNumFields()-1)
				temp += ", ";
		}
		temp += U.RPAREN + " VALUES " + U.LPAREN;
		for (int i = 0; i < sd.getNumFields(); i++) {
			Field f = sd.getField(i);
			String value = sd.getFieldValue(i, data);
			if (value == null) // This field could not be processed
				return null;

			if (f.getOutType().compareTo("CHAR") == 0)
				temp += "'" + value + "'";
			else 
				temp += value;

			if (i != sd.getNumFields()-1)
				temp += ", ";
		}
		temp += U.SPACE + U.RPAREN + U.SEMI;
		return temp;
    }

    /*CREATE EXAMPLE */
    /*
      create table employee
      (first varchar(15),
      last varchar(20),
      age number(3),
      address varchar(30),
      city varchar(20),
      state varchar(20));
    */

    public String getCreateTable(SchemaDefinition sd) {
	// SQL CREATE TABLE
	
		String temp = "CREATE TABLE " +
		    takeOutNonAlphas(sd.getTableName()) + U.SPACE + U.LPAREN;
		for (int i = 0; i < sd.getNumFields(); i++) {
			Field f = sd.getField(i);

			// Translate types into Postgres format
			String type = f.getOutType();
			if (type.compareTo("CHAR") == 0) 
			    type = "VARCHAR";
			else if (type.compareTo("INTEGER") == 0) {
			    type = "DECIMAL";
			    if (f.getMaxOutLength() > 9)
				f.setMaxOutLength(9);
			}
			else if (type.compareTo("REAL") == 0)
			    type = "DECIMAL";
			temp += f.getName() + U.SPACE;              
			temp += type + U.LPAREN + f.getMaxOutLength() + U.RPAREN;           
			if (i != sd.getNumFields()-1)
				temp += U.COMMA + U.NL;
		}
		temp += U.SPACE + U.RPAREN + U.SEMI;
		return temp;
    }                  

    public String quit() {
	return "";
    }
}
