import java.io.*;
import java.util.*;
          
public class SchemaDefiner {
// This program is a simple tool for generating schema definitions.

    static BufferedReader in;

    public SchemaDefiner() {
    }
     
    public static void main(String[] args) {
      in = new BufferedReader(new InputStreamReader(System.in));
      System.out.println("");
      System.out.println("Schema Definer v1.0");                 
      System.out.println("====================\n");
      createSchema();
    }

    private static void createSchema() {
	SchemaDefinition sd = new SchemaDefinition();
	System.out.print("What is this schema called? ");
	String name = getInput();
	System.out.print("How many fields are in the data? ");
	int numFields = getNum(getInput());
	if (numFields == 0)
	    System.exit(0);
	sd.setName(name);
	
	boolean done = false;
	for (int i = 0; i < numFields; i++) {
	    Field f = null;
	    while (!done) {
		f = new Field();   
		System.out.println("Field #" + (i+1) +":");  
		System.out.print("What is the name of this field? ");
		f.setName(getInput());
		while (f.getInType().length() == 0) {
		    System.out.println("Field types...");
		    System.out.println("1. Character");
		    System.out.println("2. Integer");
		    System.out.println("3. Real/Floating Point");
		    System.out.print("Select a field type: ");
		    int intVal = 0;
		    try {
			intVal = Integer.parseInt(getInput());
		    }
		    catch (NumberFormatException e) {};
		    if (intVal == 1) 
			f.setInType("CHAR");
		    else if (intVal == 2)
			f.setInType("INTEGER");
		    else if (intVal == 3) 
			f.setInType("REAL");
		    else
			System.out.println("Invalid type selection.");
		}
		System.out.print("What is the delimiter for this field? ");
		String del = getInput();
		f.setDelimiter(del);
		if (del == null)
		    f.setVariable(false);
		else if (del.length() == 0)
		    f.setVariable(false);
		else if (del.indexOf(" ") != -1)
		    f.setVariable(false);
		else {
		    System.out.print("(v)ariable of (f)ixed width? ");
		    String str = getInput();
		    if (str.length() == 0)
			f.setVariable(false);
		    if (str.charAt(0) == 'v')
			f.setVariable(true);           
		    else
			f.setVariable(false); 
		}
		if (f.isVariable()) { 
		    System.out.print("Min width: ");
		    f.setMinInLength(getNum(getInput()));
		    System.out.print("Max width: ");
		    f.setMaxInLength(getNum(getInput()));
		}
		else {
		    System.out.print("Length: ");
		    f.setMaxInLength(getNum(getInput()));
		    f.setMinInLength(f.getMaxInLength());
		}
		
		int validCode = f.validate();
		if (validCode >= 0)
		    done = true;
		else {
		    System.out.print("Field failed validation with code ");  
		    System.out.println(validCode + ".");
		}
	    }
	    sd.addField(f);         
	    done = false;
	}
	System.out.print("Are there any values that need to be mapped(y/n)? ");
	String needMapping = getInput();
	if ((needMapping.length() > 0) && (needMapping.charAt(0) == 'y')) {
	    getValueMapping(sd);
	}
	
	
	PrintWriter out = null;
	if (name.length() > 0) {
	    try {
		out = new PrintWriter(new FileOutputStream(name));
	    }
	    catch (IOException e) {
		e.printStackTrace();
	    }
	    out.print(sd.toXML());
	    out.close();
	}
    }
	
    private static void getValueMapping(SchemaDefinition sd) {
	boolean doneMapping = false;
	while (!doneMapping) {
	    System.out.println("Fields:");
	    for (int i = 0; i < sd.getNumFields(); i++) {
		System.out.println((i+1)+". " + sd.getField(i).getName());
	    }	
	    System.out.println((sd.getNumFields()+1) + ". Exit mapping");
	    System.out.print("Select a field to adding mapping for... ");
	    String selectedField = getInput();
	    int fieldNum = 0;
	    try {
		fieldNum = Integer.parseInt(selectedField);
	    }
	    catch (NumberFormatException e) {
		System.out.println("Invalid field selection.");
	    }
	    if (fieldNum == sd.getNumFields()+1)
		doneMapping = true;
	    if ((fieldNum > 0) && (fieldNum <= sd.getNumFields())) {
		Field f = sd.getField(fieldNum-1);
		if (!f.hasValueMapping()) {
			String outType = "";
			while (outType.length() == 0) {
			    System.out.println("Field types...");
			    System.out.println("1. Character");
			    System.out.println("2. Integer");
			    System.out.println("3. Real/Floating Point");
			    System.out.print("Select an Output type: ");
			    int intVal = 0;
			    try {
				intVal = Integer.parseInt(getInput());
			    }
			    catch (NumberFormatException e) {};
			    if (intVal == 1) 
				outType = "CHAR";
			    else if (intVal == 2)
				outType = "INTEGER";
			    else if (intVal == 3) 
				outType = "REAL";
			    else
				System.out.println("Invalid type selection.");
			    
			}
			f.setOutType(outType);
			if (f.getOutType().compareTo("CHAR") == 0) {
			    int maxOutLength = 0;
			    while (maxOutLength < 1) {
				System.out.print("What is the maximum output length? ");
				maxOutLength = getNum(getInput());
				if (maxOutLength < 1) 
				    System.out.println("The maximum output length must be greater than 0.");
			    }
			    f.setMaxOutLength(maxOutLength);
			}
			boolean ok = false;
			while (!ok) {
			    System.out.print("What is the default value? ");
			    String val = getInput();
			    if (val.length() == 0)
				ok = true;
			    else if (val.length() > f.getMaxOutLength())
				ok = false;
			    else if (f.getOutType().compareTo("INTEGER") == 0) {
				try {
				    int intVal = Integer.parseInt(val);
				    ok = true;
				}
				catch (NumberFormatException e) {
				    ok = false;
				}
			    }
			    else if (f.getOutType().compareTo("REAL") == 0) {
				try {
				    float floatVal = Float.parseFloat(val);
				    ok = true;
				}
				catch (NumberFormatException e) {
				    ok = false;
				}
			    }
			    else
				ok = true;
			    if (!ok) 
				System.out.println("The value entered was invalid.");
			    else 
				f.setDefaultValue(val);
			}
		}
		System.out.println(f.getName() + "...");
		System.out.println("In Type: " + f.getInType());
		System.out.println("Minimum Length: " + f.getMinInLength());
		System.out.println("Maximum Length: " + f.getMaxInLength());
		if (f.getOutType().compareTo("CHAR") == 0)
		    System.out.println("Maximum Output Length: " + f.getMaxOutLength());
		System.out.println("------------------------------");
		System.out.print("Enter an input value: ");
		String inVal = getInput();
		System.out.print("Enter an output value: ");
		String outVal = getInput();
		int result = f.addValueMapping(inVal, outVal);
		if (result != 0) 
		    System.out.println("The value mapping was invalid because of code " + result);
		System.out.print("Do you want to map more values(y/n)? ");
		String moreValues = getInput();
		if ((moreValues.length() > 0) && (moreValues.charAt(0) == 'y'))
		    doneMapping = false;
		else
		    doneMapping = true;
	    }
	}
    }

    private static String getInput() {
	String input = "";
	int i = 0;
	try {
	    input = in.readLine();
	}
	catch (IOException e) {
	    e.printStackTrace();
	} 
	return input;
    }
	
	
    private static int getNum(String text) {
	int num = 0;
	try {
	    num = (new Integer(text)).intValue();
	}
	catch (NumberFormatException e) {
	    //System.out.println("Invalid value.");
	}
	return num;
    }
    
    private static Vector openSchema(String fileName) {
	Vector fields = new Vector();
	SchemaDefinition sd = new SchemaDefinition(fileName);
	return sd.getFields();
    }

} 
