import java.util.*;
import java.io.*;
import org.apache.xerces.parsers.*;
import org.w3c.dom.*;
import java.lang.*;
    
public class SchemaDefinition {


    private Vector fields;
    private String name;
    private String tableName;
    private int lastFieldProcessed = 0;
    private int lastFieldEndIndex = 0;
    
    public SchemaDefinition () {
	fields = new Vector();
    }

    public SchemaDefinition(String filePath) {
	this();
	DOMParser p = new DOMParser();
	
	try { p.parse(filePath); }
	catch (IOException e) { 
	    e.printStackTrace();
	}
	catch (org.xml.sax.SAXException e2) { e2.printStackTrace(); }
	Document doc = p.getDocument();
	NodeList top = doc.getChildNodes();
	if (top.getLength() == 0)
	    return;
	if (top.item(1).getNodeName().compareTo("SchemaDefinition") != 0)
	    return;
	
	NamedNodeMap attrs = top.item(1).getAttributes();
	for (int i = 0; i < attrs.getLength(); i++) {
	    Node attr = attrs.item(i);
	    String nodeName = attr.getNodeName();
	    if (nodeName.compareTo("name") == 0)
		setName((String)attr.getNodeValue());
	    else if (nodeName.compareTo("tableName") == 0) {
		String tb = (String)attr.getNodeValue();
		setTableName(tb);
	    }
	}
	
	NodeList topNodes = top.item(1).getChildNodes();
	for (int i = 0; i < topNodes.getLength(); i++) {
	    String nodeName = topNodes.item(i).getNodeName();
	    if (nodeName.compareTo("Field") == 0)
		processField(topNodes.item(i));
	}
    }
    
    private void processField(Node fieldNode) {
	NamedNodeMap attrs = fieldNode.getAttributes();
	Field f = new Field();
	
	for (int i = 0; i < attrs.getLength(); i++) {
	    Node attr = attrs.item(i);
	    String nodeName = attr.getNodeName();
	    if (nodeName.compareTo("name") == 0)
		f.setName((String)attr.getNodeValue());
	    else if (nodeName.compareTo("isVariable") == 0) 
		f.setVariable((new Boolean(attr.getNodeValue())).booleanValue());
	    else if (nodeName.compareTo("minInLength") == 0)
		f.setMinInLength(Integer.parseInt(attr.getNodeValue()));
	    else if (nodeName.compareTo("maxInLength") == 0)
		f.setMaxInLength(Integer.parseInt(attr.getNodeValue()));
	    else if (nodeName.compareTo("maxOutLength") == 0)
		f.setMaxOutLength(Integer.parseInt(attr.getNodeValue()));
	    else if (nodeName.compareTo("delimiter") == 0)
		f.setDelimiter((String)attr.getNodeValue());
	    else if (nodeName.compareTo("inType") == 0)
		f.setInType((String)attr.getNodeValue());
	    else if (nodeName.compareTo("outType") == 0)
		f.setOutType((String)attr.getNodeValue());
	    else if (nodeName.compareTo("isIncluded") == 0)
		f.setIncluded((new Boolean(attr.getNodeValue())).booleanValue());
	    else if (nodeName.compareTo("defaultValue") == 0)
		f.setDefaultValue((String)attr.getNodeValue());
	}
	NodeList values = fieldNode.getChildNodes();
	for (int i = 0; i < values.getLength(); i++) {
	    Node value = values.item(i);
	    String inVal = "";
	    String outVal = "";
	    NamedNodeMap valAttrs = value.getAttributes();
	    for (int j = 0; j < valAttrs.getLength(); j++) {
		Node attr = valAttrs.item(j);
		String attrName = attr.getNodeName();
		if (attrName.compareTo("inVal") == 0)
		    inVal = attr.getNodeValue();
		else if (attrName.compareTo("outVal") == 0)
		    outVal = attr.getNodeValue();
	    }
	    f.addValueMapping(inVal, outVal);
	}
	fields.addElement(f);
    }

    public String getName() {
	return name;
    }
    
    public void setName(String name) {
	this.name = name;
    }
    
    public String getTableName() {
	return tableName;
    }
    
    public void setTableName(String tableName) {
	this.tableName = tableName;
    }
    
    public Vector getFields() {
	return fields;
    }
    
    public int getNumFields() {
	return fields.size();
    }
    
    public Field getField(int index) {
	if (fields.size() == 0)
	    return null;
	
	return (Field)fields.elementAt(index);
    }
    
    public void setField(int index, Field field) {
	if (fields.size() == 0)
	    return;
	if (index < 0)
	    return; 
	if (index >= fields.size())
	    return;
	
	fields.setElementAt(field, index);
    }
    
    public int addField(Field field) {
	fields.addElement(field);
	return fields.size()-1;
    }
    
    public Field removeField(int index) {
	if (fields.size() == 0)
	    return null;
	if (index >= fields.size())
	    return null;
	if (index < 0)
	    return null;
	
	return (Field)fields.remove(index);
    }
    
    public String getFieldValue(int fieldIndex, String data) {
	
	if (fieldIndex >= getNumFields())
	    return null;
	
	String value = null;
	int startIndex = 0;
	int endIndex = 0;
	int fieldsToFind = fieldIndex;
	int currentFieldIndex = 0;
	if (lastFieldProcessed == fieldIndex-1) {
	    startIndex = lastFieldEndIndex +
		getField(lastFieldProcessed).getDelimiter().length();
	    currentFieldIndex = fieldIndex;
	}
	
	Field f = null;
	while (currentFieldIndex <= fieldIndex) {
	    try {
		f = getField(currentFieldIndex);
		if (currentFieldIndex == getNumFields()-1) { // This is the last field
			//if (f.isVariable())
		    value = data.substring(startIndex, data.length());
		    //		    System.out.println("Before: " + value);
		    if (!f.isVariable() && (value.length() > f.getMaxOutLength
()))
			value = value.substring(0, f.getMaxOutLength());
			//else
			//	value = data.substring(startIndex, startIndex + f.getMaxInLength());
		    //System.out.println("After: " + value);
			}
		else if ((f.getDelimiter().length() == 0) || (!f.isVariable())) {
		    endIndex = startIndex + f.getMaxInLength();
		    value = data.substring(startIndex, endIndex);
		}
		else {
		    endIndex = data.indexOf(f.getDelimiter(), startIndex);
		    if (endIndex == -1)
			return null;
		    
		    int len = endIndex - startIndex;
		    if ((len > f.getMaxInLength()) || (len < f.getMinInLength()))
			return null;
		    
		    value = data.substring(startIndex, endIndex);
		}
		currentFieldIndex++;
	    }
	    catch (Exception e) {
		return null;
	    }
	    if ((f.isVariable()) && (value.length() < f.getMaxInLength()))
		return null;
	    if (value.length() > f.getMaxInLength())
		return null;
	}

	value = value.trim();
	//	System.out.println(value);
	if (f.getInType().compareTo("INTEGER") == 0) {
	    try {
		int i = Integer.parseInt(value);
	    }
	    catch (NumberFormatException e) {
		return null;
	    }
	}
	else if (f.getInType().compareTo("FLOAT") == 0) {
	    try {
		float fl = Float.parseFloat(value);
	    }
	    catch (NumberFormatException e) {
		return null;
	    }
	}
	
	if (f.hasValueMapping()) {
	    value = f.getMappedValue(value);
	    if (value == null)
		value = f.getDefaultValue();
	    else if (value.length() == 0)
		value = f.getDefaultValue();
	    else if (value.length() > f.getMaxOutLength())
		return null;
	}
	lastFieldProcessed = fieldIndex;
	lastFieldEndIndex = endIndex;
	value = value.trim();
	if (value.length() > f.getMaxOutLength())
	    return null;
	return value;
    }
    
    public String toXML() {
	String temp = "";
	temp += "<?xml version=\"1.0\"?>";
	temp += "<!DOCTYPE SchemaDefinition SYSTEM"; 
	temp += " \"SchemaDefinition.dtd\">";
	temp += "<SchemaDefinition name=\"" + name + "\" tableName=\"" + tableName + "\">";
	for (int i = 0; i < fields.size(); i++) {
	    Field f = (Field)fields.elementAt(i);
	    temp += f.toXML();
	}
	temp += "</SchemaDefinition>";
	return temp;
    }

}
